import os,os.path,string
import cherrypy
from Common import config,DetConfig
import pluginCondDBPyInterface as CondDB
from CondCore.Utilities import iovInspector as inspect
import simplejson as json

class IOV(object):
    def __init__(self,configfilepath):
        self.__config=config.Config()
        self.__config.from_ini(configfilepath)
        self.__authpath=self.__config.common.auth_path
        self.__datasavedir=self.__config.common.datasave_dir
        self.__detconfig=DetConfig.DetConfig(self.__config.common.detector_conf)
        self.__cmsframe=CondDB.FWIncantation()
        #self.__logdb=str(self.__config.iov.log_conn)
        self.__rdbms=CondDB.RDBMS(self.__authpath)
    @cherrypy.expose
    def index(self):
        """Top level page
        """
        overviewstr=['IOV inspection service']
        overviewstr.append(self.__authpath)
        overviewstr.append(self.__datasavedir)
        return (' ').join(overviewstr)   
    
    def __getconnectstr(self,det,service,schema,viacache):
        """Assemble full connect string with given parameters
        """
        connect_str=''
        if not viacache:
            if (service.find(":"))==-1 :
               connect_str+='oracle://'
            connect_str+=service+'/'+schema          
        else:            
            #knownservices=self.__detconfig.services(det)
            #if '/'.join([service,schema]) not in knownservices:
            #   return ''
            servletname=self.__detconfig.serviceToServlet(det,service)
            connect_str+='frontier://(serverurl=http://cmsfrontier.cern.ch:8000/'+servletname+')/'+schema
        if len(connect_str)==0:
            return "Application Server Error: requested service "+service+" "+schema+"is not configured"
        return connect_str
    
    @cherrypy.expose
    def getTags(self,det,service,schema,viacache=0):
        """Get list of tags
        """
        result=''
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
            return json.dumps('Application Server Error: '+service+' '+schema+' is not configured' )
        try:
            db=self.__rdbms.getDB(connect_str)
            tags=db.allTags()
            result=tags
        except Exception, er:
            json.dumps('Application Server Error: '+str(er))
        except RuntimeError:
            json.dumps('Application Server RuntimeError')
        except:
            json.dumps('Application Server Unknown Error')
        return json.dumps(result)
    
    @cherrypy.expose
    def getPayloadSummariesForTag(self,det,service,schema,tag,since=0,till=0,head=0,tail=0,viacache=0):
        """Get payload summary info for given tag in the given iov range
        """
        result={"status":"",
                "data":[]
               }
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        try:
            db = self.__rdbms.getDB(connect_str)
            result["data"]=inspect.Iov(db,tag).payloadSummaries()
            result["status"]="OK"
        except Exception, er:
            result["status"]=str(er)
        except RuntimeError:
            result["status"]="RuntimeError"
        except:
            result["status"]="Unknown Error"
        return json.dumps(result)
    
    @cherrypy.expose
    def getIOVForTag(self,det,service,schema,tag,since=0,till=0,head=0,tail=0,viacache=0):
        """Get detailed iov info for given tag in the given range
        """
        result=''
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
            return json.dumps('Application Server Error: '+service+' '+schema+' is not configured')
        try:
            db = self.__rdbms.getDB(connect_str)
            v=inspect.Iov(db,tag)
            result={"comment":"",
                "revision":"",
                "payloadContainerName":"",
                "timetype":"",
                "data":[]}
            result['comment']=v.comment()
            result['revision']=v.revision()
            result['payloadContainerName']=v.payloadContainerName()
            result['timetype']=str(v.timetype())            
            suma=v.list()
            for s in suma:
              r={}
              r["payloadToken"]=s[0]
              r["since"]=s[1]
              r["till"]=s[2]
              result["data"].append(r)
        except Exception, er:
            json.dumps('Application Server Error: '+str(er))
        except RuntimeError:
            json.dumps('Application Server RuntimeError')
        except:
            json.dumps('Application Server Unknown Error')
        return json.dumps(result)

    @cherrypy.expose
    def getWhatForTag(self,det,service,schema,tag,viacache):
        """get What() information defined by the payload
        """
        result=''
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
           return json.dumps('Application Server Error: '+service+' '+schema+' is not configured')
        try:
            db = self.__rdbms.getDB(connect_str)
            result=inspect.extractorWhat(db,tag)
        except Exception, er:
            json.dumps('Application Server Error: '+str(er))
        except RuntimeError:
            json.dumps('Application Server RuntimeError')
        except:
            json.dumps('Application Server Unknown Error')
        return json.dumps(result)
 

    @cherrypy.expose
    def getTrendForTag(self,det,service,schema,tag,since=0,till=0,extractor=0,viacache=0):
        """get trend data with the given trend extractor for a given tag in the given range. This is an example. Detector should parse or treat its trend data accordingly in the detector specific modules
        """
        result=''
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        e={}
        if extractor != 0:
          try:
            e=json.loads(extractor)
          except ValueError,er:
            return json.dumps("data loading error "+er)
        if len(connect_str)==0:
            return json.dumps("Application Server Error: requested service "+service+" "+schema+"is not configured")
        try:
            db = self.__rdbms.getDB(connect_str)
            result=inspect.Iov(db,tag).trend(e)
        except Exception, er:
            print str(er)
            return json.dumps(str(er))
        except RuntimeError:
            print 'RuntimeError'
            return json.dumps("RuntimeError")
        except:
            print 'unknownerror'
            return json.dumps('unknown error')
        return json.dumps(result)

